//
//  DetailViewController.m
//  AnalogSynthesizerPad
//
//  Created by 成田 拓郎 on 10/07/19.
//  Copyright __MyCompanyName__ 2010. All rights reserved.
//

#import "DetailViewController.h"
#import "RootViewController.h"


@interface DetailViewController ()
@property (nonatomic, retain) UIPopoverController *popoverController;
@end



@implementation DetailViewController

@synthesize toolbar, popoverController;
@synthesize keyboardView;
@synthesize volume;
@synthesize synthesizerOutput;

#pragma mark -
#pragma mark Managing the detail item

/*
 RootViewが選択された場合の処理
 */
- (void)selectRootView:(int)row {
	switch (row) {
		case 0:	//Organ
			break;
		case 1:	//Piano
			break;
		case 2:	//Lead Guitar
			break;
		case 3:	//Strings
			break;
		default:
			break;
	}

    if (popoverController != nil) {
        [popoverController dismissPopoverAnimated:YES];
    }        
}

#pragma mark -
#pragma mark Split view support

- (void)splitViewController: (UISplitViewController*)svc willHideViewController:(UIViewController *)aViewController withBarButtonItem:(UIBarButtonItem*)barButtonItem forPopoverController: (UIPopoverController*)pc {
    
    barButtonItem.title = @"Type List";
    NSMutableArray *items = [[toolbar items] mutableCopy];
    [items insertObject:barButtonItem atIndex:0];
    [toolbar setItems:items animated:YES];
    [items release];
    self.popoverController = pc;
}


// Called when the view is shown again in the split view, invalidating the button and popover controller.
- (void)splitViewController: (UISplitViewController*)svc willShowViewController:(UIViewController *)aViewController invalidatingBarButtonItem:(UIBarButtonItem *)barButtonItem {
    
    NSMutableArray *items = [[toolbar items] mutableCopy];
    [items removeObjectAtIndex:0];
    [toolbar setItems:items animated:YES];
    [items release];
    self.popoverController = nil;
}


#pragma mark -
#pragma mark Rotation support

// Ensure that the view controller supports rotation and that the split view can therefore show in both portrait and landscape.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    return YES;
}

- (void)willAnimateRotationToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation
										 duration:(NSTimeInterval)duration {
	if (toInterfaceOrientation == UIInterfaceOrientationPortrait ||
		toInterfaceOrientation == UIInterfaceOrientationPortraitUpsideDown) {
		// ポートレート モード
		self.volume.center = CGPointMake(64, 110);
		self.volume.bounds = CGRectMake(0, 0, 48, 48);
	}
	else {
		// ランドスケープ モード
		self.volume.center = CGPointMake(640, 110);
		self.volume.bounds = CGRectMake(0, 0, 48, 48);
	}
}

#pragma mark -
#pragma mark View lifecycle

//オーディオセッション　割り込みリスナーコールバック関数
static void interruptionListenerCallback(void *inUserData, UInt32 interruptionState)
{
	DetailViewController *controller = (DetailViewController *)inUserData;
	
	if (interruptionState == kAudioSessionBeginInterruption)
	{
		//オーディオセッション割り込みの開始
		//再生の停止
		controller.synthesizerOutput.play = NO;
	}
	else if (interruptionState == kAudioSessionEndInterruption)
	{
		//オーディオセッション割り込みの終了
		//オーディオセッションのカテゴリを設定
		UInt32 sessionCategory = kAudioSessionCategory_SoloAmbientSound;
		AudioSessionSetProperty(kAudioSessionProperty_AudioCategory,
								sizeof(sessionCategory),
								&sessionCategory);
		//オーディオセッションのアクティブ化
		AudioSessionSetActive(true);
		//再生の再開
		controller.synthesizerOutput.play = YES;
	}
}

// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad {
    [super viewDidLoad];

	[keyboardView setTarget:self];
	[keyboardView setAction:@selector(keyboardChanged:)];
	[volume setTarget:self];
	[volume setAction:@selector(volumeChanged:)];
	volume.valueKnob = 0;  // ボリュームの値
	volume.valueMax = 127; // ボリュームの最大値
	[volume rotateView];   // 設定した値を反映させる

	//オーディオセッションを初期化
	AudioSessionInitialize(NULL, NULL, interruptionListenerCallback, self);
	//オーディオセッションのカテゴリを設定
	UInt32 sessionCategory = kAudioSessionCategory_SoloAmbientSound;
	AudioSessionSetProperty(kAudioSessionProperty_AudioCategory,
							sizeof(sessionCategory),
							&sessionCategory);
	//バッファサイズを設定
	Float32 bufferSize = 0.02f; // 0.02秒
	AudioSessionSetProperty(kAudioSessionProperty_PreferredHardwareIOBufferDuration,
							sizeof(bufferSize),
							&bufferSize);
	//オーディオセッションのアクティブ化
	AudioSessionSetActive(true);
	
	//AudioUnitの準備
	SynthesizerOutput *synthe = [[SynthesizerOutput alloc] init];
	self.synthesizerOutput = synthe;
	[synthe release];

	// 再生開始
	self.synthesizerOutput.play = YES;
}

/*
- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
}
*/
/*
- (void)viewDidAppear:(BOOL)animated {
    [super viewDidAppear:animated];
}
*/
/*
- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:animated];
}
*/
/*
- (void)viewDidDisappear:(BOOL)animated {
    [super viewDidDisappear:animated];
}
*/

- (void)viewDidUnload {
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
    self.popoverController = nil;
	self.keyboardView = nil;
	self.volume = nil;
}


#pragma mark -
#pragma mark Memory management

/*
- (void)didReceiveMemoryWarning {
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Release any cached data, images, etc that aren't in use.
}
*/

- (void)dealloc {
    [popoverController release];
    [toolbar release];

	[keyboardView release];
	[volume release];
	[synthesizerOutput release];

    [super dealloc];
}

- (void)keyboardChanged:(id)sender {
//	NSLog(@"Keyboard Message");
	synthesizerOutput.keyboard = keyboardView.keyDown;
}

- (void)volumeChanged:(id)sender {
//	NSLog(@"Volume Message");
}

@end
